# Author: Ivan Canay | Northwestern University | January 2016
#-------------------------------------------------------------------
# X-plain: Functions for Inference under covariate-adaptive Randomization with Expensive Treatment
#------------------------------------------------------------------

require("compiler")
require("MASS")
library(exactRankTests)
library(coin)
require("sandwich")
require("lmtest")
require("VGAM")

# We use the R compilter to improve the speed of the functions.
#-------------------------------------------------------------------
# List of Functions:
# gen.baseline.one: generates potential outcomes and 1 covariate
# form.strata.one:  forms strata in the case of 1 covariate
# dgp.srs: Random assignments using SRS
# dgp.wei: Random assignments using Wei's design
# dgp.bcd: Random assignments using Bias Coin Design
# dgp.sbr:Random assignments using permuted blocks within strata
# t.stat: computes the absolute value of the two-sample t-statistic
# t.stat2 and t.nume compute two sample t-stat and the numerator of it
# P.test: Computes a standard permutation test
# CAP.test Computes the covariate adaptive permutation (CAP) test
# true.values: computes all the sigmas and other stuff of the true model
#------------------------------------------------------------------

#-------------------------------------------------------------------
gen.baseline.one <- function(n,model,theta,gamma,sg1){
#-------------------------------------------------------------------
# X-plain: Generates covariates (W) and potential outcomes (Y.1,Y.0)
# using different functions m() and epsilons.
#-------------------------------------------------------------------
  mu.1<- theta; # We normalized mu.0=0 so theta=mu.1

  if (model==1){ #linear model with normal epsi and beta W
    # This model introduces high conservativeness of t-test
    # Here both SFE and SAT should work just fine
    epsi.1<- sqrt(sg1)*rnorm(n); # sg1: sigma squared epsi 1
    epsi.0<- rnorm(n);
    W     <- sqrt(20)*(rbeta(n,2,2)-1/2); # recentered 0 and re-scaled 1
    # function m() of each potential outcomes (check E[m(W)]=0)
    m0    <- gamma*W;
    m1    <- m0;
    Y.1   <- mu.1 + m1 + epsi.1;
    Y.0   <- m0 + epsi.0;
  }
   if (model==2){ #linear/non-linear but heterogeneous 
    # This model with sg1=2 and pi=1/3 should introduce distortion for HOMO-SFE
    # If gamma=4 sigmaH becomes small. gamma=1 or 2 are fine. 
    epsi.1<- sqrt(sg1)*rnorm(n); # sg1: sigma squared epsi 1
    epsi.0<- rnorm(n);
    W     <- sqrt(20)*(rbeta(n,2,2)-1/2); # recentered 0 and re-scaled 1
    # function m() of each potential outcomes (check E[m(W)]=0)
    m1    <- gamma*W;
    m0    <- gamma*(-((log(W+3))*(W<1/2)-0.5602138));
    Y.1   <- mu.1 + m1 + epsi.1;
    Y.0   <- m0 + epsi.0;
  }
   if (model==3){ #Like model 2 but heteroskedastic
    W     <- sqrt(20)*(rbeta(n,2,2)-1/2); # recentered 0 and re-scaled 1
    epsi.1<- sqrt(abs(W))*sqrt(sg1)*rnorm(n); # sg1: sigma squared epsi 1
    epsi.0<- sqrt(abs(W))*rnorm(n);
     # function m() of each potential outcomes (check E[m(W)]=0)
    m1    <- gamma*W;
    m0    <- gamma*(-((log(W+3))*(W<1/2)-0.5602138));
    Y.1   <- mu.1 + m1 + epsi.1;
    Y.0   <- m0 + epsi.0;
  }
  if (model==4){ #Non-linear, t-distribution, homogeneous.
    W     <- runif(n,-2,2);
    epsi.1<- abs(W)*sqrt(sg1)*rt(n,3)/3; # sg1: sigma squared epsi 1
    epsi.0<- abs(W)*rt(n,3)/3; # divide by 3 to make var = 1.
    # function m() of each potential outcomes (check E[m(W)]=0)
    m0    <- gamma*(W*(W+1<0)+W^2*(W+1>=0)*(W<=1)+W*(W>1)-0.1724343);
    m1    <- gamma*(W^2*(W+1<0)+W*(W+1>=0)*(W<=1)+W^2*(W>1)-1.158188);
    Y.0   <- m0 + epsi.0;
    Y.1   <- mu.1 + m1 + epsi.1;
  }

  # List of returns
  list(Y.0=Y.0,Y.1=Y.1,W=W,e0=epsi.0,e1=epsi.1,m.1=m1,m.0=m0,n=n,model=model)
}
#-------------------------------------------------------------------
form.strata.one <- function(baseline,num.strata){
#-------------------------------------------------------------------
# X-plain: Generates strata indicators from covariates (W).
#         - W: covariates (must be scalar)
#         - num.strata: the number of strata to be formed
#         - model: model in baseline.
#-------------------------------------------------------------------
model<-baseline$model;
n <- baseline$n;

if (model<=3){# W is beta in [-sqrt(20)/2,sqrt(20)/2]
  W<-baseline$W;
  bounds<-seq(-2.25,2.25,length.out = num.strata+1); # careful with bounds
  I.S   <- matrix(0,n,num.strata);
  for (s in 1:num.strata){
    I.S[,s]<- (W>bounds[s])*(W<=bounds[s+1]);
  }     
}
if (model==4){# W is uniform [-2,2]
  W<-baseline$W;
  bounds<-seq(-2,2,length.out = num.strata+1); # careful with bounds
  I.S   <- matrix(0,n,num.strata);
  for (s in 1:num.strata){
    I.S[,s]<- (W>bounds[s])*(W<=bounds[s+1]);
  }     
}


return(I.S)
}
#-------------------------------------------------------------------
dgp.srs <- function(baseline,I.S,pi=1/2){
#-------------------------------------------------------------------
# X-plain: Generates treatment assignments and outcomes using SRS.
#         - baseline: structure from `gen.baseline' 
#         - I.S: matrix from `form.strata' 
#         - pi: target proportion of n1/n
#-------------------------------------------------------------------
  n   <-baseline$n;
  num.strata <- ncol(I.S);
  A   <-(runif(n)<=pi); # treatment Assignment
  AS  <- A*I.S;

  if (pi==0){
     A   =cbind(rep(0,n));    # treatment Assignment
    l.seq=num.strata/2;  
    v.pi=c(seq(0.2,0.40,(0.40-0.2)/(l.seq-1)),seq(0.60,0.8,(0.8-0.6)/(l.seq-1))); 
    for (k in 1:num.strata){
    index   <- which(I.S[,k]==1);
    ns      <- length(index);
    A[index]<- (runif(ns)<=v.pi[k]);
  }
  AS <- matrix(c(A)*I.S,n,num.strata);
}

  Y   <- baseline$Y.1*A + baseline$Y.0*(1-A);

  data<-data.frame(Y=Y,A=A,W=baseline$W,I.S=I.S,AS=AS);
  return(data);
}
#-------------------------------------------------------------------
dgp.wei <- function(baseline,I.S,pi=1/2){
#-------------------------------------------------------------------
# X-plain: Generates treatment assignments and outcomes using Wei's design.
#         - baseline: structure from `gen.baseline' 
#         - I.S: matrix from `form.strata' 
#         - pi: target proportion of n1/n
#-------------------------------------------------------------------
  num.strata <- ncol(I.S);
  n   <-baseline$n;
  A   <- cbind(rep(0,n));    # treatment Assignment
  D.s <- rep(0,num.strata);  # inbalance in each stratum s
  n.s <- rep(0,num.strata);  # sample size in each s
	v.pi = rep(pi,each=num.strata);
  if (pi==0){
    half.1 = floor(num.strata/2);
    half.2 = num.strata-half.1;
    v.pi=c(rep(0.3,each=half.1),rep(0.7,each=half.2));
  }

	for (j in 1:n){
    index   <- which(I.S[j,]==1);
    n.s[index]<-n.s[index]+1;
    phi     <-(1-D.s[index]/n.s[index])*v.pi[index];
    A[j]    <-(runif(1)<=phi);
    D.s[index]<-(1/v.pi[index])*sum(A*(I.S[,index]==1))-n.s[index]; 
	}

  Y   <- baseline$Y.1*A + baseline$Y.0*(1-A);

  A.S <- matrix(c(A)*I.S,n,num.strata);

  data<-data.frame(Y=Y,A=A,W=baseline$W,I.S=I.S,AS=A.S);
  return(data);
}
#-------------------------------------------------------------------
dgp.bcd <- function(baseline,I.S,pi=1/2,factor=1/2){
#-------------------------------------------------------------------
# X-plain: Treatment assignments and outcomes using Bias Coin design.
#         - baseline: structure from `gen.baseline' 
#         - I.S: matrix from `form.strata' 
#         - pi: target proportion of n1/n
#-------------------------------------------------------------------
	num.strata <- ncol(I.S);
  n   <-baseline$n;
  A   <- cbind(rep(0,n));    # treatment Assignment
  D.s <- rep(0,num.strata);  # inbalance in each stratum s
  n.s <- rep(0,num.strata);  # sample size in each s
  if (pi>=0.5){lambda<- pi + (1-pi)*factor};
  if (pi<0.5){lambda <- 1-pi + pi*factor};
  
  for (j in 1:n){
    index   <- which(I.S[j,]==1);
    n.s[index]<-n.s[index]+1;
    phi  <- lambda*(D.s[index]<0)+pi*(D.s[index]==0)+(1-lambda)*(D.s[index]>0);
    A[j]    <-(runif(1)<=phi);
    D.s[index]<-(1/pi)*sum(A*(I.S[,index]==1))-n.s[index]; 
  }

  Y   <- baseline$Y.1*A + baseline$Y.0*(1-A);

  A.S <- matrix(c(A)*I.S,n,num.strata);

  data<-data.frame(Y=Y,A=A,W=baseline$W,I.S=I.S,AS=A.S);
  return(data);

}
#-------------------------------------------------------------------
dgp.sbr <- function(baseline,I.S,pi=1/2){
#-------------------------------------------------------------------
# X-plain: Treatment assignments and outcomes using PBWS.
#         - baseline: structure from `gen.baseline' 
#         - I.S: matrix from `form.strata' 
#         - pi: target proportion of n1/n
#-------------------------------------------------------------------
  num.strata <- ncol(I.S);
  n   <-baseline$n;
  A   <- cbind(rep(0,n));    # treatment Assignment
  l.seq<-num.strata/2;

  v.pi = rep(pi,each=num.strata);
  if (pi==0){
    v.pi=c(seq(0.2,0.40,(0.40-0.2)/(l.seq-1)),seq(0.60,0.8,(0.8-0.6)/(l.seq-1)));
  }
   
  for (k in 1:num.strata){
    index   <- which(I.S[,k]==1);
    ns      <- length(index);
    # pick a random permutation of 1s and 0s. 
    A[index]<- sample(c(rep(1,floor(v.pi[k]*ns)),rep(0,ns-floor(v.pi[k]*ns))));
  }

  Y   <- baseline$Y.1*A + baseline$Y.0*(1-A);

  A.S <- matrix(c(A)*I.S,n,num.strata);

  data<-data.frame(Y=Y,A=A,W=baseline$W,I.S=I.S,AS=A.S);
  return(data);
}

#-------------------------------------------------------------------
t.stat <- function(data,theta=0){
#-------------------------------------------------------------------
# X-plain: compute the usual t-statistic of diff in means
#-------------------------------------------------------------------
Y   <-data$Y;
A   <-data$A;
n 	<- length(A);
n.1	<- sum(A); 
n.0	<- n-n.1;

Y1.bar   <-sum(Y*A)/n.1; 
Y0.bar   <-sum(Y*(1-A))/n.0;
sigma2_1 <-sum((Y-Y1.bar)^2*A)/n.1;
sigma2_0 <-sum((Y-Y0.bar)^2*(1-A))/n.0;

t.stat<-abs(Y1.bar-Y0.bar-theta)/sqrt(sigma2_1/n.1+sigma2_0/n.0);
}

#-------------------------------------------------------------------
t.stat2 <- function(data,theta=0){
#-------------------------------------------------------------------
# X-plain: compute the usual t-statistic of diff in means
#-------------------------------------------------------------------
Y   <-data$Y;
A   <-data$A;
n   <- length(A);
n.1 <- sum(A); 
n.0 <- n-n.1;

Y1.bar   <-sum(Y*A)/n.1; 
Y0.bar   <-sum(Y*(1-A))/n.0;
sigma2_1 <-sum((Y-Y1.bar)^2*A)/n.1;
sigma2_0 <-sum((Y-Y0.bar)^2*(1-A))/n.0;

t.stat<-(Y1.bar-Y0.bar-theta)/sqrt(sigma2_1/n.1+sigma2_0/n.0);
}

#-------------------------------------------------------------------
t.nume <- function(data,theta=0){
#-------------------------------------------------------------------
# X-plain: compute the usual t-statistic of diff in means
#-------------------------------------------------------------------
Y   <-data$Y;
A   <-data$A;
n   <- length(A);
n.1 <- sum(A); 
n.0 <- n-n.1;

Y1.bar   <-sum(Y*A)/n.1; 
Y0.bar   <-sum(Y*(1-A))/n.0;

t.nume<-sqrt(n)*(Y1.bar-Y0.bar-theta);
}


#-------------------------------------------------------------------
pre.P.test <- function(data,theta=0,M=500,alpha=0.05){
#-------------------------------------------------------------------
# X-plain: compute the usual permutation test for t-statistic
#         - data: entire data (covariates, treatment assigment, outcomes) 
#         - theta: value of theta under the null
#         - M: number of elements from the set of all permutations.  
#-------------------------------------------------------------------
# First compute Observed test statistic
  A   <-data$A; Y   <-data$Y;
  n   <- length(A);
  n.1 <- sum(A); 
  n.0	<- n-n.1;
  Y1.bar   <-sum(Y*A)/n.1; 
  Y0.bar   <-sum(Y*(1-A))/n.0;
  sigma2_1 <-sum((Y-Y1.bar)^2*A)/n.1;
  sigma2_0 <-sum((Y-Y0.bar)^2*(1-A))/n.0; 
  
  ObsT<-abs(Y1.bar-Y0.bar-theta)/sqrt(sigma2_1/n.1+sigma2_0/n.0);
  
  # Now Compute permuted test statistic. First get all transformations
  ind   <- (runif(n*M)>1/2);
  ind   <- matrix(ind,n,M);
  A.per <- abs(A-ind); # every column is 1 permutation
  Y1.per <- Y*A.per; # every column is 1 permutation
  Y0.per <- Y*(1-A.per); # every column is 1 permutation
  n.1    <- colSums(A.per); n.0<-n-n.1; # number of ones and zeros per column
  bar_1  <- colSums(Y1.per)/n.1; # average per set of permutations
  bar_0  <- colSums(Y0.per)/n.0;
  vec.bar_1 <-t(matrix(rep(bar_1,n),M,n)); # useful to compute variance
  vec.bar_0 <-t(matrix(rep(bar_0,n),M,n));
  
  sig.sq_1   <- colSums((Y1.per-vec.bar_1)^2*A.per)/n.1;
  sig.sq_0   <- colSums((Y0.per-vec.bar_0)^2*(1-A.per))/n.0;
    
  # Compute the new test statistic over permuted data
  nume    <- abs(bar_1-bar_0-theta);
  deno    <- sqrt(sig.sq_1/n.1 + sig.sq_0/n.0);
  NewT    <- sort(nume/deno);    # sort the vector of Test Stat  
  k       <- M-floor(M*alpha);   # find index for quantile
  Mplus   <- sum(NewT>NewT[k]);  # M+ from LR book
  M0      <- sum(NewT==NewT[k]); # M0 from LR book
  prob    <- (M*alpha-Mplus)/M0; # term a(x) from LR book
  # randomized test 
  rule <- (ObsT>NewT[k]) + (ObsT==NewT[k])*(runif(1)<=prob);  
  # non-randomized test
  Nrule <- (ObsT>NewT[k]);
  ties <- sum(ObsT==NewT[k]);
  
  # List of returns
  list(stat=ObsT,rule=rule, Nrule=Nrule, cv=NewT[k],ties=ties)  
}
#-------------------------------------------------------------------
P.test <- cmpfun(pre.P.test);
#-------------------------------------------------------------------

#-------------------------------------------------------------------
pre.CAP.test <- function(data,theta=0,alpha=0.05,M=500){
#-------------------------------------------------------------------
# X-plain: Covariate Adaptive Permutation test. 
#         - data: entire data (covariates, treatment assigment, outcomes) 
#         - theta: value of theta under the null
#         - M: number of elements from the set of all permutations.  
#-------------------------------------------------------------------
  I.S   <-data[,grepl("I.S", names(data))]
  num.strata<- ncol(I.S);
  Y   <-data$Y;
  A   <-data$A;
  n   <- length(A);
  ObsT<-t.stat(data,theta);
  
  # Now Compute the set of CAP permutations
  A.per      <- matrix(0,n,M); 
   
  for (k in 1:num.strata){
    index   <- which(I.S[,k]==1);
    A.s     <-A[index];
    # Here use "replicate" to get a n x M matrix of Permutations.
    if (length(index)>0){
        A.per[index,]<- replicate(M,sample(A.s));
    }
  }
  # include the identity element
  A.per[,1]<-A;
  
  # Now Compute permuted test statistic. First get all transformations
  Y1.per <- Y*A.per; # every column is 1 permutation
  Y0.per <- Y*(1-A.per); # every column is 1 permutation
  n.1    <- colSums(A.per); n.0<-n-n.1; # number of ones and zeros per column
  bar_1  <- colSums(Y1.per)/n.1; # average per set of permutations
  bar_0  <- colSums(Y0.per)/n.0;
  vec.bar_1 <-t(matrix(rep(bar_1,n),M,n)); # useful to compute variance
  vec.bar_0 <-t(matrix(rep(bar_0,n),M,n));
  
  sig.sq_1   <- colSums((Y1.per-vec.bar_1)^2*A.per)/n.1;
  sig.sq_0   <- colSums((Y0.per-vec.bar_0)^2*(1-A.per))/n.0;
    
  # Compute the new test statistic over permuted data
  nume    <- abs(bar_1-bar_0-theta);
  deno    <- sqrt(sig.sq_1/n.1 + sig.sq_0/n.0);
  NewT    <- sort(nume/deno);    # sort the vector of Test Stat  
  k       <- M-floor(M*alpha);   # find index for quantile
  Mplus   <- sum(NewT>NewT[k]);  # M+ from LR book
  M0      <- sum(NewT==NewT[k]); # M0 from LR book
  prob    <- (M*alpha-Mplus)/M0; # term a(x) from LR book
  # randomized test 
  rule <- (ObsT>NewT[k]) + (ObsT==NewT[k])*(runif(1)<=prob);  
  # non-randomized test
  Nrule <- (ObsT>NewT[k]);
  ties <- sum(ObsT==NewT[k]);
  
  # List of returns
  list(stat=ObsT,rule=rule, Nrule=Nrule, cv=NewT[k],ties=ties)  
  
}
#-------------------------------------------------------------------
CAP.test <- cmpfun(pre.CAP.test);
#-------------------------------------------------------------------

#-------------------------------------------------------------------
true.values <- function(model,gamma,sg1,num.strata,pi){
#-------------------------------------------------------------------
# X-plain: true values of the DGP
#-------------------------------------------------------------------
  NN<-100000; # Sample size to compute varsigmas. 
  base.BIG  <- gen.baseline.one(NN,model,0,gamma,sg1);
  I.S       <-form.strata.one(base.BIG,num.strata);
  naI.S       <- I.S; # here we replace zeros with NA
  naI.S[naI.S==0]<-NA
  Y.0 <-base.BIG$Y.0;
  Y.1 <-base.BIG$Y.1;

  # varsigma Y
  v.sg.Y <- var(Y.1)/pi + var(Y.0)/(1-pi); 
  v.sg.YB <- var(Y.1)/(1-pi) + var(Y.0)/pi;

  # varsigma tilde Y
  cond.mean.0 <- colMeans(Y.0*naI.S,na.rm=TRUE);
  cond.mean.1 <- colMeans(Y.1*naI.S,na.rm=TRUE);
  Vec.mean.0  <- rowSums(matrix(rep(cond.mean.0,each=NN),nrow=NN)*naI.S,na.rm=TRUE); 
  Vec.mean.1  <- rowSums(matrix(rep(cond.mean.1,each=NN),nrow=NN)*naI.S,na.rm=TRUE);
  tilde.Y.0   <- base.BIG$Y.0 - Vec.mean.0;
  tilde.Y.1   <- base.BIG$Y.1 - Vec.mean.1;
  v.sg.tilde.Y <- var(tilde.Y.1)/pi + var(tilde.Y.0)/(1-pi);
  v.sg.tilde.YB<- var(tilde.Y.1)/(1-pi) + var(tilde.Y.0)/pi;

  # varsigma H
  p.s <- colMeans(I.S);
  C.Mean.m0 <- colMeans(base.BIG$m.0*naI.S,na.rm=TRUE);
  C.Mean.m1 <- colMeans(base.BIG$m.1*naI.S,na.rm=TRUE);
  v.sg.H    <- sum(p.s*(C.Mean.m1-C.Mean.m0)^2);
  # varsigma A: this is zero for BCD and SBR
  v.sg.A.srs <- pi*(1-pi)*sum(p.s*(C.Mean.m1/pi+C.Mean.m0/(1-pi))^2);
  v.sg.A.srsB<- pi*(1-pi)*sum(p.s*(C.Mean.m1/(1-pi)+C.Mean.m0/pi)^2);
  v.sg.A.wei <- (1/3)*v.sg.A.srs;
  v.sg.A.weiB <- (1/3)*v.sg.A.srsB;
  # varsigma pi: This appears in sfe without saturation
  v.sg.pi.srs <- (1-2*pi)^2/(pi*(1-pi))*sum(p.s*(C.Mean.m1-C.Mean.m0)^2);
  # variance Numerator of t-statistics
  num        <- v.sg.tilde.Y+v.sg.H+v.sg.A.srs;
  
  list(VY1=var(Y.1),VY0=var(Y.0),VTY1=var(tilde.Y.1),VTY0=var(tilde.Y.0),
      vsgY=v.sg.Y,vsgTY=v.sg.tilde.Y,vsgH=v.sg.H,vsgA=v.sg.A.srs);
}

#-------------------------------------------------------------------------------------------------
sat.car<-function(fit,data,cluster="name.cluster",n.treat){
#----------------------------------------------------------------------------------------------
# X-plain: Computes point estimators and variance covariance matrix of SAT estimator under CAR
#         The method assumes tau(s)=0 for all s (say, stratified block randomization)
#     - fit: the output of a lm SAT regression
#       - data: the data set
#       - cluster: the name of the variable defining the clusters
#       - n.treat: number of treatments
#----------------------------------------------------------------------------------------------
# Parameters
S = star_sw[[cluster]]; # here we choose the cluster variable using a string name
n.obs= fit$df.residual+length(fit$coefficients)
n.strata=nlevels(factor(S));
reg.levels = unlist(fit$xlevels);
reg.strata = length(reg.levels);
n.coef=n.treat*n.strata;

# Check if there are factors lost in the regression (and redefine S if needed)
if (n.strata!=reg.strata){
  unique.S    = unique(factor(na.omit(S)))
  missing.level = unique(unique.S[! unique.S %in% reg.levels]);
  S=S[S!=missing.level];
  n.strata=nlevels(factor(S));
}

# coefficients and weights. 
weights=summary(factor(na.omit(S)))/sum(summary(factor(na.omit(S))));
betas=coef(fit);
v.betas=matrix(betas,n.strata,(n.treat+1))
# Compute theta ignoring NAs
beta.noNA=v.betas;
beta.noNA[is.na(beta.noNA)]=0;
w.noNA = weights;
w.noNA[is.na(w.noNA)]=0;
thetas = w.noNA%*%beta.noNA;
v.thetas=matrix(thetas,nrow=n.strata,ncol=length(thetas),byrow=TRUE);

# Estimate the new standard errors
inner = (v.betas-v.thetas)[,-1];
n.na = length(which(is.na(inner)));
R.cols=n.strata+(n.treat*n.strata)-n.na;
R.matrix=matrix(0,n.treat,R.cols);
u.bound = n.strata;

# Do loop over strata being careful with NAs : dim(R.matrix) must be compatible with dim (HC.matrix)
VH=matrix(0,n.treat,n.treat);

for (s in 1:n.strata){
  # first create the matrix VH
  if (n.treat==1){
    vector=cbind(inner[s]);
    } else {vector=cbind(inner[s,]);}
  n.na = length(which(is.na(vector)));
  vector[is.na(vector)]=0;
  VH = VH + weights[s]*vector%*%t(vector)
  # then work on the matrix R needed to get VY
  R.submat= diag(n.treat-n.na)*weights[s];
  l.bound = u.bound+1; 
  u.bound = l.bound+n.treat-1-n.na;
  R.matrix[,l.bound:u.bound]= R.submat;
}

HC.matrix=vcovHC(fit,type="HC1");
VY = n.obs*diag(diag(R.matrix%*%HC.matrix%*%t(R.matrix)),n.treat);

A.VAR = VH + VY;

se = sqrt(diag(A.VAR/n.obs));
se.hc = sqrt(diag(VY/n.obs));

t.test = thetas[-1]/se;
p.value= 100*2*(1-pnorm(t.test));
t.HC   = thetas[-1]/se.hc;
p.HC   = 100*2*(1-pnorm(t.HC));

# List of returns
list(theta=thetas[-1],t.test.new=t.test,p.value.new=p.value, t.test.HC=t.HC,
 p.value.HC=p.HC,VAR=A.VAR,n.obs=n.obs,n.coef=n.coef,n.strata=n.strata)
} 
#-------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------
send.mail<-function(addr,subject="Mail from R",text="empty text"){
#-------------------------------------------------------------------
# X-plan: Sends Notification Email
    mail.cmd<-paste("mail ",
                    "-s \"",subject,"\" ",
                    addr,
                    " << EOT &\n",
                    text,"\n",
                    "EOT",
                    sep="",collapse="")
     system(mail.cmd,intern=FALSE)
}
